------------------------------------------------------------------------
-- Copyright (C) 2002 Daniel Heck
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--  
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--  
-- You should have received a copy of the GNU General Public License along
-- with this program; if not, write to the Free Software Foundation, Inc.,
-- 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
------------------------------------------------------------------------


---------------
-- FUNCTIONS --
---------------
function progress(percent)
    local fontname = "levelmenu"
    local scr      = video.GetScreen();
    local d        = scr:get_drawable()

    d:box(0,0,scr:width(), scr:height(), d:map_color(0,0,0))
    
    local logo = enigma.GetImage("enigma_logo2")
    local x = (scr:width()  - logo:width())/2
    local y = (scr:height() - logo:height())/2
    d:blit(x , y-logo:height(), logo)
    d:box(x, y, logo:width()*percent/100, 2,
          d:map_color(130,150,150))
    scr:update_all();
    scr:flush_updates();
end


modeltag = 0
function unique_modelname()
    modeltag = modeltag + 1
    return "xmodel" .. modeltag
end

-- Define a new image model.  'name' is the name of the model being created
-- and 'opt' contains optional information about the image file.  The following
-- fields can be set in 'opt':
--
--     * filename
--     * xoff,yoff (hotspot coordinates inside the image)
--
-- Suitable default values are used for all options
function def_image(name, opt)
    opt = opt or {}
    name = name or unique_modelname()
    fname = (opt.filename or name)
    opt = opt or {}
    display.DefineImage(name, fname, opt.xoff or 0, opt.yoff or 0)
    return name
end

-- Define multiple image models at once.  Use the same options for all
-- of them.
function def_images(names, opt)
    opt = opt or {}
    for i,name in names do
	def_image(name,opt)
    end
end

-- Define many image models from one single big image file.

function def_subimages(name, options)
    local opts = options or { }
    local w = opts.w or 1
    local h = opts.h or 1
    local imgw = opts.imgw or 32
    local imgh = opts.imgh or 32
    local xoff = opts.xoff or 0
    local yoff = opts.yoff or 0
    local modelname = opts.modelname or name

    local imagelist={}
    local cnt = 1
    local r=Rect:new(0,0,0,0)
    for x=1,w do
        for y=1,h do
            r.x,r.y,r.w,r.h = imgw*(x-1),imgh*(y-1),imgw,imgh
            tinsert(imagelist, modelname..cnt)
            display.DefineSubImage(modelname..cnt, name, xoff,yoff, r)
            cnt = cnt+1
        end
    end
    r:delete()
    return imagelist
end

function def_stone(name, shmodel, opt)
    opt = opt or {}
    shmodel = shmodel or "sh-solid"
    opt.filename = opt.filename or name
    display.DefineShadedModel(name, def_image(nil, opt), shmodel)
end

function def_stones(names)
    for i,name in names do def_stone(name) end
end

-- Define a shaded model named `name' with texture model `fg' and
-- shadow model `bg'.  Both are real models (not names of image files), so 
-- you can combine animated images with shadows etc.
function def_shmodel(name, fg, bg)
    display.DefineShadedModel(name, fg, bg)
end

function def_overlay(name, imglist)
    display.DefineOverlayImage(name, getn(imglist), imglist)
end

function def_solidstone(name, front)
    def_shmodel(name, front, "sh-solid")
end

function def_alias(name, othername)
    display.DefineAlias(name,othername)
end

----------------
-- ANIMATIONS --
----------------

-- Generate a list of frame names by appending increasing numerical
-- prefixes to a base name.  For example, 'framenames("hello", 1,2)'
-- yields {"hello_0001", "hello_0002"}.
-- [Filenames like this are created by Gimp's "Video/Split Image.."
-- tool.]

function framenames(prefix, first, last)
    local fn = {}
    for i=first,last do
	tinsert(fn, prefix .. format("_%04d", i))
    end
    return fn
end

-- Build a list of frames from a list of model names and a constant
-- duration.
function buildframes(names, msec)
    local a={}
    for i,n in names do a[i] = {n, msec} end
    return a
end

-- Build a list of frames from (1) a list of names and (2) a list of
-- frame durations.  These two list are assumed to have the same
-- number of entries, or, more generally, to have the same index set.
function composeframes(namelist, mseclist)
    local a={}
    for i=1,getn(namelist) do a[i] = {namelist[i], mseclist[i]} end
    return a
end


-- Given a list of frames, this function generates a new framelist
-- with for a ping-pong style animation.  (For example, an "abcd"
-- style animation would result in an "abcddcba"-style one.)

function pingpong(framelist)
    local a=framelist
    local n=getn(framelist)
    for i = 0,n-1 do
	a[n+i+1] = framelist[n-i]
    end
    return a
end

function repeat_frames(framelist, blocksize, cnt)
    local a={}
    for i=1,getn(framelist),blocksize do
	for j=1,cnt do
	    for k=i,i+blocksize-1 do
		tinsert(a,framelist[k])
	    end
	end
    end
    return a
end

function repeatanim(framelist)
    return repeat_frames(framelist, getn(framelist), 2)
end

function reverseframes(framelist)
    local a={}
    for i=getn(framelist),1,-1 do
	tinsert(a, framelist[i])
    end
    return a
end

-- Define an animation from a list of images.  `frames' is a
-- framelist, as built with `framenames()', but the model names in
-- this list are interpreted as image filenames.

function def_anim_images(name, frames, opt)
    opt = opt or {}
    display.DefineAnim(name, opt.loop)
    for i=1,getn(frames) do
	local frame=frames[i]
	opt.filename = frame[1]
	local immodel = def_image(nil, opt)
	display.AddFrame(name, immodel, frame[2])
    end
end


-- Define an animation from a list of models.

function def_anim(name, frames, loop)
    display.DefineAnim(name, loop)
    for i=1,getn(frames) do
	local frame = frames[i]
	display.AddFrame(name, frame[1], frame[2])
    end
end

function NewAnim(name,opts) -- name, img, h,w,speed,pingpong, loop)
    local imagefile = opts.img or name
    local h=opts.h or 1
    local w=opts.w or 1
    local speed = opts.speed or 100
    local loop=opts.loop or 0

    local frames = def_subimages(imagefile, {["h"]=h, ["w"]=w})
    if opts.pingpong then frames=pingpong(frames) end
    def_anim(name, buildframes(frames, speed), loop)
end

    
------------------------------------------------------------------------
-- MODELS
------------------------------------------------------------------------

----------------------
-- General settings --
----------------------
def_image("invisible")

function print_table(t)
    for i,v in t do
        if type(v)=="table" then
            print (i.." - "..v[1]..","..v[2])
        else
            print (i.." - "..v)
        end
    end
end


--------------------------------------------------------------------
--                           ACTOR MODELS                         --
--------------------------------------------------------------------
progress(0)

----------------------
-- Small white ball --
----------------------
do
    local fg,img

    -- Normal
    def_image("sh-whiteball-small", {xoff=-3, yoff=-3})
    fg=def_image(nil, {filename="ac-whiteball-small", xoff=-6, yoff=-6})
    def_shmodel("ac-whiteball-small", fg, "sh-whiteball-small")

    -- Appearing
    --def_anim("ac-whiteball-small-appear", 
    --         {{"invisible", 30},{"ac-whiteball-small", 50}})

    -- Falling
    img=def_subimages("ac-whiteball-small-fall", 
                      {modelname="ac-wbs-f",h=5,imgw=12,imgh=12,
                          xoff=-6,yoff=-6})
    tinsert(img, "invisible")
    def_anim("ac-whiteball-small-fall", composeframes(img,{70,65,60,55,50,30}))
    def_alias("ac-whiteball-small-fallen", "invisible")

    -- Appearing
    def_anim("ac-whiteball-small-appear", reverseframes(buildframes(img, 30)))
    -- Shattering
    img=def_subimages("ac-whiteball-small-shatter",
                      {h=5, imgw=20, imgh=20, xoff=-10, yoff=-10})
    def_anim("ac-whiteball-small-shatter", buildframes(img, 60))
    def_image("ac-whiteball-small-shattered", {xoff=-8, yoff=-9})
end

----------------
-- Black ball --
----------------
do
    local img,f
    
    -- Normal
    def_image("sh-blackball", {xoff=-6, yoff=-6})
    def_image("fg-bb", {filename="ac-blackball", xoff=-9, yoff=-9})
    def_shmodel("ac-blackball", "fg-bb", "sh-blackball")

    -- Falling
    img=def_subimages("ac-blackball-fall",{h=10, imgw=24, imgh=24, xoff=-12, yoff=-12})
    tinsert(img, "invisible")
    f=composeframes(img,{70,65,60,55,50,50,50,50,50,50,50})
    def_anim("ac-blackball-fall", f)
    def_alias("ac-blackball-fallen", "invisible")

    -- Appearing
    def_anim("ac-blackball-appear", reverseframes(buildframes(img, 30)))

    -- Shattering
    img=def_subimages("ac-blackball-shatter",
                      {h=5, imgw=40, imgh=40, xoff=-20, yoff=-20})
    def_anim("ac-blackball-shatter", buildframes(img, 60))

    def_image("ac-blackball-shattered", {xoff=-20, yoff=-20})
end

----------------
-- White ball --
----------------
do
    local img,f
    
    -- Normal
    def_image("sh-whiteball", {xoff=-6, yoff=-6})
    def_image("fg-wb", {filename="ac-whiteball", xoff=-9, yoff=-9})
    def_shmodel("ac-whiteball", "fg-wb", "sh-whiteball")

    -- Falling
    img=def_subimages("ac-whiteball-fall",{h=10, imgw=24, imgh=24, xoff=-12, yoff=-12})
    tinsert(img, "invisible")
    f=composeframes(img,{70,65,60,55,50,50,50,50,50,50,50})
    def_anim("ac-whiteball-fall", f)
    def_alias("ac-whiteball-fallen", "invisible")

    -- Appearing
    def_anim("ac-whiteball-appear", reverseframes(buildframes(img, 30)))

    -- Shattering
    img=def_subimages("ac-whiteball-shatter",
                      {h=5, imgw=40, imgh=40, xoff=-20, yoff=-20})
    def_anim("ac-whiteball-shatter", buildframes(img, 60))

    def_alias("ac-whiteball-shattered", img[5])
end



-- Shattering black ball --


-- Jumping black ball --
namelist=def_subimages("ac-blackball-jump", {h=4, imgw=32, imgh=32, xoff=-16, yoff=-16})
shadows=def_subimages("sh-blackball-jump", {h=4, imgw=32, imgh=32, xoff=-10, yoff=-10})
frames={}
for i=1,4 do
    def_shmodel("bb-jump"..i, namelist[i], shadows[i])
    tinsert(frames, "bb-jump"..i)
end
def_anim("ac-blackball-jump",
         pingpong(buildframes(frames, 70)))


-- Black marble in inventory  --
def_image("inv-blackball")


------------------
-- Floor models --
------------------
progress(10)

function def_floors(floorlist)
    for i,name in floorlist do
	def_image(name)
    end
end

function def_randfloor(name, imagelist)
    def_floors(imagelist)
    display.DefineRandModel(name, getn(imagelist), imagelist)
end

-- _si for _subimage
function def_randfloor_si(name, height, width)
    imagelist=def_subimages(name, {h=height, w=width})
    display.DefineRandModel(name, getn(imagelist), imagelist)
end

if options.WizardMode > 0 then
    def_subimages("fl-gradient-wiz", {modelname="fl-gradient",w=6, h=4})
else
    def_subimages("fl-gradient", {w=6, h=4})
end

def_image("fl-rock", {filename="fl-rock1"})
def_floors{"fl-abyss"}
def_floors{"fl-dunes", "fl-sand", "fl-bluegreen", "fl-bluegreenx"}
def_floors{"fl-inverse"}
def_image("fl-normal")
def_randfloor_si("fl-hay", 4)
--def_randfloor("fl-hay",{"fl-hay1", "fl-hay2"})
def_floors{"fl-floor_001"}
def_floors{"fl-ice_001"}
def_floors{"fl-stone"}

def_randfloor_si("fl-brick", 3)
def_randfloor_si("fl-space", 3,2)
def_randfloor_si("fl-rough", 5)
def_randfloor_si("fl-gray", 5)
def_randfloor_si("fl-leaves", 3)
def_randfloor_si("fl-metal", 5)
def_randfloor_si("fl-plank", 4)
def_randfloor_si("fl-water", 4)
def_randfloor_si("fl-woven", 5)
def_randfloor_si("fl-marble", 4)
def_randfloor_si("fl-wood", 4)
def_randfloor_si("fl-bluegray", 4)
--
-- Bridges
--
def_image("fl-bridgea-open")
def_image("fl-bridgea-closed")
namelist=def_subimages("fl-bridgea", {h=10})
frames = buildframes(namelist,70)
def_anim("fl-bridgea-opening", reverseframes(frames))
def_anim("fl-bridgea-closing", frames)


def_image("fl-bridgex-open")
def_image("fl-bridgex-closed")
namelist=def_subimages("fl-bridgex", {h=9})
frames = buildframes(namelist,70)
def_anim("fl-bridgex-opening", frames)
def_anim("fl-bridgex-closing", reverseframes(frames))


def_image("fl-bridgey-open")
def_image("fl-bridgey-closed")
namelist=def_subimages("fl-bridgey", {h=9})
frames = buildframes(namelist,70)
def_anim("fl-bridgey-opening", frames)
def_anim("fl-bridgey-closing", reverseframes(frames))

-----------------
-- Item models --
-----------------
progress(20)

def_images{
    "it-blackbomb",
    "it-brush",
    "it-document", 
    "it-dynamite", 
    "it-extralife",
    "it-floppy",
    "it-hammer",
    "it-hill", "it-tinyhill", 
    "it-hollow", "it-tinyhollow",
    "it-key",
    "it-laserh", "it-laserv", "it-laserhv",
    "it-magicwand", 
    "it-spade",
    "it-spring1", "it-spring2",
    "it-sword",
    "it-trigger", "it-trigger1",
    "it-umbrella",
    "it-yinyang",
}

def_alias("it-key_a", "it-key")
def_alias("it-key_b", "it-key")
def_alias("it-key_c", "it-key")


-- Cracks --
namelist=def_subimages("it-crack", {h=8})
--frames=buildframes({"it-crack4", "it-crack6", "it-crack7", "it-crack8"},90)
frames=buildframes(namelist,50)
def_anim("it-debris", frames)

-- Magnet --
def_image("it-magnet-off")
frames=buildframes(def_subimages("it-magnet-on", {h=5}), 100)
def_anim("it-magnet-on", frames, 1)

-- Wormhole --
def_images{"it-wormhole"}

-- Coins --
def_images{"it-coin", "it-coin1", "it-coin2", "it-coin4"}

-- Burning dynamite --
dyn_frames=buildframes(def_subimages("it-dynamite-burning", {h=15}), 100)
def_anim("it-dynamite-burning", dyn_frames) --repeat_frames(dyn_frames,3,2))

-- Burning bomb --
frames=buildframes(def_subimages("it-blackbomb-burning", {h=9}), 100)
def_anim("it-blackbomb-burning", frames) --repeat_frames(frames,3,2))

-- Explosion --
def_anim_images("expl", {{"expl", 00}})

-- Pipes --
def_subimages("it-pipe", {h=2, w=5})

-- Shogun dots --

NewAnim("it-shogun1", {img="it-shogun-small", h=3, speed=160, pingpong=1, loop=1})
NewAnim("it-shogun2", {img="it-shogun-med",   h=3, speed=160, pingpong=1, loop=1})
NewAnim("it-shogun3", {img="it-shogun-big",   h=3, speed=160, pingpong=1, loop=1})

------------------
-- STONE MODELS --
------------------
progress(30)

-- stone shades
def_image("sh-solid")
def_image("sh-grate1")
def_image("sh-grate2")
def_image("sh-glass")
def_image("sh-white4")
def_image("sh-puzzle1")

-- stone models
if (video.GetColorDepth()==8) then
    def_stone("st-glass", "sh-glass", {filename="st-glass8"})
else
    def_stone("st-glass", "sh-glass");
end
def_stone("st-rock1")
def_stone("st-rock2")
def_stone("st-rock3")
def_stone("st-rock4")
def_stone("st-brownie")

def_alias("st-rock3_hole", "st-rock3")
def_alias("st-rock3_move", "st-rock3")

def_stone("st-marble")
def_alias("st-marble_hole", "st-marble")
def_alias("st-marble_move", "st-marble")

def_stone("st-wood")
def_stone("st-grate1", "sh-grate1")
def_stone("st-grate2", "sh-grate2")
def_stone("st-swap")
def_stone("st-bombs", nil, {filename="st-bombs2"})

def_stone("st-greenbrown")
def_alias("st-greenbrown_hole", "st-greenbrown")
def_alias("st-greenbrown_move", "st-greenbrown")

def_stone("st-stone_break", nil, {filename="st-stone_break"})
def_stone("st-brick")
def_stone("st-woven")
def_stone("st-brick_magic", nil, {filename="st-brick"})
def_stone("st-switch0", nil, {filename="st-switch2"})
def_stone("st-switch1", nil, {filename="st-switch1"})
def_stone("st-floppy0", nil, {filename="st-floppy1"})
def_stone("st-floppy1", nil, {filename="st-floppy2"})

def_stone("st-laserswitch0", nil, {filename="st-oxydb"})
def_anim_images("laserswitch-blink",
                repeatanim(pingpong
                           (buildframes(framenames("st-fakeoxyd-blink", 1,4),
                            50))), {loop=1})
def_solidstone("st-laserswitch1", "laserswitch-blink")

def_stone("st-wood_001")
def_stone("st-key0", nil, {filename="st-key1"})
def_stone("st-key1", nil, {filename="st-key2"})
def_stone("st-black1")
def_stone("st-black2")
def_stone("st-black3")
def_stone("st-black4", "sh-white4")
def_stone("st-white1")
def_stone("st-white2")
def_stone("st-white3")
def_stone("st-white4", "sh-white4")
def_stone("st-yinyang1")
def_stone("st-yinyang2")
def_stone("st-bluegray")
def_stone("st-plain")
def_stone("st-yellow")


------------------
-- Death's head --
------------------
do
    def_stone("st-death")
    local img=def_subimages("st-death-munch", {h=3})
    def_anim("st-death-munch-anim", buildframes(img, 140))
    def_solidstone("st-death-munch", "st-death-munch-anim")
end

-----------------
-- Oxyd Stones --
-----------------

-- Oxyd stones occur in different colors and different flavors and
-- for each combination of these two attributes we need animations
-- for the following internal states or state transitions:
--
--   (a) opening and closing
--   (b) blinking
--   (c) opened
--
-- Creating the required image files by hand would be tedious, so
-- we use a couple of specialized functions and overlays to create 
-- them automatically.

do
    local colorspots = framenames("st-oxydbtempl", 2,9)
    local openovls = framenames("st-oxydbtempl", 10,14)

-- Define "fading in" and "fading out" animations for oxyd stones.
-- These two animations are combined with the stone images to 
-- produce the opening and closing animations for oxyd stones.

    local baseimg = {
        a="st-oxyda-open", 
        b="st-oxydb-open",
        c="st-oxydc-open",
        d="st-oxydd-open"
    }
    local fopening = {
        a = buildframes(def_subimages("st-oxyda-opening", {h=9}), 60),
        b = buildframes(def_subimages("st-oxydb-opening", {h=14}), 40),
        c = buildframes(def_subimages("st-oxydc-opening", {h=5}), 70),
        d = buildframes(def_subimages("st-oxydd-opening", {h=5}), 70),
    }
    local fclosing = {
        a = reverseframes(fopening["a"]),
        b = reverseframes(fopening["b"]),
        c = reverseframes(fopening["c"]),
        d = reverseframes(fopening["d"]),
    }

    function mkopenclose(flavor, color)
        local n = "st-oxyd" .. flavor .. color
        local fadein = "oxyd"..flavor.."-fadein"
        local fadeout= "oxyd"..flavor.."-fadeout"
        
        def_overlay(n.."-base", {%baseimg[flavor], %colorspots[color+1]})
        def_solidstone(n, n.."-base")
        display.DefineComposite(n.."-opening", n, fadein)
        display.DefineComposite(n.."-closing", n, fadeout) 
    end

    function mkblink(flavor, color)
        local n = "st-oxyd"..flavor..color.."-blink"
        local img={%baseimg[flavor],%colorspots[color+1], "st-oxyd-questmark"}
        def_overlay(n..1, img)
        def_overlay(n..2, {%baseimg[flavor], %colorspots[color+1]})
        def_anim(n.."-anim", buildframes({n..1,n..2}, 500), 1)
        def_shmodel(n, n.."-anim", "sh-solid")
    end

    function mkopened(flavor, color)
        local n = "st-oxyd" .. flavor .. color .. "-open"
        local names = {}
        
        for i=1,getn(%openovls) do
            local images={%baseimg[flavor],%colorspots[color+1],%openovls[i]}
            names[i] = n .. format("_%04d", i)
            def_overlay(names[i], images)
        end
        
        -- compose these images into an animation
        frames = pingpong(buildframes(names, 100))
        def_anim(n.."-anim", frames, 1)
        
        -- and finally add a shadow to make the model complete
        def_solidstone(n, n.."-anim")
    end
    
    function mkoxyd(flavor)
        --    print ("making oxyd model for "..flavor)
        def_stone("st-oxyd"..flavor)
        
        local fadein = "oxyd"..flavor.."-fadein"
        local fadeout= "oxyd"..flavor.."-fadeout"
        def_anim(fadein, %fopening[flavor])
        def_anim(fadeout, %fclosing[flavor])
        
        for color=0,7 do
            mkopenclose(flavor, color)
            mkblink(flavor, color)
            mkopened(flavor, color)
        end
    end
    mkoxyd("a")
    mkoxyd("b")
    progress(40)
    mkoxyd("d")
    mkoxyd("c")
end

progress(50)

-----------------
-- Laser stone --
-----------------
function make_laser(dir)
    laseron=framenames("st-laser"..dir, 1, 9)

    -- deactivated laser
    def_overlay("laser"..dir, {"st-laser-base", laseron[1]})
    def_solidstone("st-laser"..dir, "laser"..dir)

    -- activated laser
    names = {}
    for i=1,getn(laseron) do
	names[i] = "st-laseron"..dir .. format("_%04d", i)
	def_overlay (names[i], {"st-laser-base", laseron[i]})
    end
    frames = buildframes(names, 100)
    def_anim("st-laseron-anim"..dir, frames, 1)
    def_solidstone("st-laseron"..dir, "st-laseron-anim"..dir)
end

make_laser("-e")
make_laser("-s")
make_laser("-w")
make_laser("-n")

---------------------
-- Fake oxyd stone --
---------------------
def_stone("st-fakeoxyd", nil, {filename="st-oxydb"})
def_anim_images("fakeoxyd-blink",
                repeatanim(pingpong
                           (buildframes(framenames("st-fakeoxyd-blink", 1,4), 
                                        50))))
def_solidstone("st-fakeoxyd-blink", "fakeoxyd-blink")

-- Fart stone
def_stone("st-fart", nil, {filename="st-oxydb"})
def_anim_images("farting",
                repeatanim(pingpong
                           (buildframes(framenames("st-fakeoxyd-blink", 1,4), 
                                        50))))
def_solidstone("st-farting", "farting")

-- Actor impulse stone
namelist = def_subimages("st-stoneimpulse", {h=5})

def_solidstone("st-actorimpulse", namelist[2])
def_anim("actorimpulse-anim", pingpong(buildframes(namelist, 50)))
def_solidstone("st-actorimpulse-anim", "actorimpulse-anim")

-- Stone impulse stone
def_solidstone("st-stoneimpulse", namelist[1])
def_anim("stoneimpulse-anim1", buildframes(namelist, 50))
def_solidstone("st-stoneimpulse-anim1", "stoneimpulse-anim1")
def_anim("stoneimpulse-anim2", reverseframes(buildframes(namelist, 50)))
def_solidstone("st-stoneimpulse-anim2", "stoneimpulse-anim2")

namelist = def_subimages("st-stoneimpulse-hollow", {h=5})

def_solidstone("st-stoneimpulse-hollow", namelist[1])
def_anim("stoneimpulse-hollow-anim1", buildframes(namelist, 50))
def_solidstone("st-stoneimpulse-hollow-anim1", "stoneimpulse-hollow-anim1")
def_anim("stoneimpulse-hollow-anim2", reverseframes(buildframes(namelist, 50)))
def_solidstone("st-stoneimpulse-hollow-anim2", "stoneimpulse-hollow-anim2")

-----------------
-- Thief stone --
-----------------
do
    local img = def_subimages("st-thief", {h=7})
    def_anim("thief-anim", pingpong(buildframes(img, 80)))
    def_solidstone("st-thief", img[1])
    def_solidstone("st-thief-anim", "thief-anim")
end

-----------------
-- Timer stone --
-----------------
do
    local img = def_subimages("st-timer", {h=4})
    def_anim("timer-anim", buildframes(img, 120), 1)
    def_solidstone("st-timer", "timer-anim")
    def_stone("st-timeroff")
end

progress(60)

-------------
-- Mirrors -- 
-------------

-- naming scheme for mirror models: 
--
--	st-{3mirror,pmirror}-{m,s}{o,t}[1234]
--
-- {m,s} -> movable or static
-- {o,t} -> opaque or transparent
-- 
-- The numbers map to actual orientations as follows:
--
--   NUMBER    TRIANG.M.   PLANE M.
--	1	south	    "-"
--	2	west	    "\"
--	3	north	    "|"
--	4	east	    "/"

function make_mirror(basename, baseimg, overlays)
    for i=1,4 do
	mname = basename .. i
	def_overlay (mname .. "-ovl", {baseimg, overlays[i]})
	def_shmodel(mname, mname .. "-ovl", "sh-solid")
    end
end

mirror3_opaque = framenames("st-mirrortempl", 1, 4)
mirror3_transp = framenames("st-mirrortempl", 5, 8)
mirrorp_opaque = framenames("st-mirrortempl", 9, 12)
mirrorp_transp = framenames("st-mirrortempl", 13, 16)

make_mirror("st-3mirror-mo", "st-mirror-movable", mirror3_opaque)
make_mirror("st-3mirror-so", "st-mirror-static",  mirror3_opaque)
make_mirror("st-3mirror-mt", "st-mirror-movable", mirror3_transp)
make_mirror("st-3mirror-st", "st-mirror-static",  mirror3_transp)

make_mirror("st-pmirror-mo", "st-mirror-movable", mirrorp_opaque)
make_mirror("st-pmirror-so", "st-mirror-static",  mirrorp_opaque)
make_mirror("st-pmirror-mt", "st-mirror-movable", mirrorp_transp)
make_mirror("st-pmirror-st", "st-mirror-static",  mirrorp_transp)

-- OneWay --
def_stones{"st-oneway-n", "st-oneway-e", "st-oneway-w", "st-oneway-s"}

---------------
-- Coin slot --
---------------
do
    def_stone("st-coinslot")
    local img=def_subimages("st-coin2slot", {h=20})
    def_anim("st-coin2slot-anim", buildframes(img, 20))
    def_solidstone("st-coin2slot", "st-coin2slot-anim")
end


progress(70)

-------------------
-- Puzzle stones --
-------------------
do
    def_subimages("st-puzzle", {modelname="st-puzzlex",w=4,h=4})
    for i=2,16 do def_solidstone("st-puzzle"..i, "st-puzzlex"..i) end
    def_shmodel("st-puzzle1", "st-puzzlex1", "sh-puzzle1")
end

----------
-- Door --
----------
do
    local f,img

    img=def_subimages("st-doorh", {h=7})
    def_alias("st-doorh-open", img[1])
    def_alias("st-doorh-closed", img[7])
    f=buildframes(img, 60)
    def_anim("st-doorh-opening", reverseframes(f))
    def_anim("st-doorh-closing", f)

    img=def_subimages("st-doorv", {w=7})
    def_alias("st-doorv-open", img[1])
    def_alias("st-doorv-closed", img[7])
    f = buildframes(img,60)
    def_anim("st-doorv-opening", reverseframes(f))
    def_anim("st-doorv-closing", f)
end

-- Door_a --
do
    def_alias("st-door_a-open", "st-grate1")
    def_alias("st-door_a-closed", "st-oxyda")
    local f = buildframes({"st-door_a-closed", "st-door_a-open"},60)
    def_anim("st-door_a-opening", f)
    def_anim("st-door_a-closing", reverseframes(f))
end

-- Door_b --
do
    def_alias("st-door_b-open", "invisible") 
    def_alias("st-door_b-closed", "st-plain")
    local img=def_subimages("st-doorb", {modelname="doorb", h=8})
    local f = buildframes(img,60)
    def_anim("st-door_b-opening", f)
    def_anim("st-door_b-closing", reverseframes(f))
end

-- Door_c --
do
    def_alias("st-door_c-open", "st-grate2")
    def_alias("st-door_c-closed", "st-plain")
    frames=buildframes({"st-door_c-closed","st-door_c-open"},60)
    def_anim("st-door_c-opening", frames)
    def_anim("st-door_c-closing", reverseframes(frames))
end

-------------------
-- Shogun stones --
-------------------
do
    def_images{"sh-shogun1","sh-shogun2", "sh-shogun4"}
    def_subimages("st-shogun", {modelname="st-shogun-fg",h=7})

    def_shmodel("st-shogun1", "st-shogun-fg1", "sh-shogun1")
    def_shmodel("st-shogun2", "st-shogun-fg2", "sh-shogun2")
    def_shmodel("st-shogun3", "st-shogun-fg3", "sh-shogun1")
    def_shmodel("st-shogun4", "st-shogun-fg4", "sh-shogun4")
    def_shmodel("st-shogun5", "st-shogun-fg5", "sh-shogun1")
    def_shmodel("st-shogun6", "st-shogun-fg6", "sh-shogun2")
    def_shmodel("st-shogun7", "st-shogun-fg7", "sh-shogun1")
end

progress(80)

-------------------
-- Bolder stones --
-------------------
do
    local img=def_subimages("st-bolder", {w=4,h=3})
    function def_bolder(orient, start)
	local animname="st-bolder"..orient.."a"
	local frames={%img[start], %img[start+1], %img[start+2]}
	def_anim(animname, buildframes(frames, 120),1)
	def_shmodel("st-bolder-"..orient, animname, "sh-solid")
    end
    
    def_bolder("n",1)
    def_bolder("e",4)
    def_bolder("s",7)
    def_bolder("w",10)
end

-- Invisible stone --
def_alias("st-invisible", "invisible")
def_alias("st-invisible_magic", "invisible")
def_alias("st-stonebrush", "invisible")

----------------------
-- Magic stones :-) --
----------------------
do
    local img = def_subimages("st-magic", {h=4, modelname="st-magic-fg"})
    local nlist = {}
    for i=1,getn(img) do 
        nlist[i] = "st-magic"..i
        def_solidstone(nlist[i], img[i]) 
    end
    display.DefineRandModel("st-magic", getn(nlist), nlist)
end


-------------
-- Effects --
-------------
progress(100)

do
    local f = buildframes(framenames("ring-anim", 1,8), 50)
    def_anim_images("ring-anim", f, {xoff=-16,yoff=-16})
end
